mod gen;

use bevy::{prelude::*, sprite::MaterialMesh2dBundle};

#[global_allocator]
static ALLOCATOR: cap::Cap<std::alloc::System> =
	cap::Cap::new(std::alloc::System, 1024 * 1024 * 1024);

fn main() {
	App::new()
		.add_plugins((
			DefaultPlugins,
			bevy_fps_counter::FpsCounterPlugin,
			bevy_pancam::PanCamPlugin,
		))
		.add_systems(Startup, setup)
		.configure_sets(Update, (Set::Demo).chain())
		.add_systems(Update, (demo_system.in_set(Set::Demo),))
		.run();
}

fn setup(
	mut commands: Commands,
	mut meshes: ResMut<Assets<Mesh>>,
	mut materials: ResMut<Assets<ColorMaterial>>,
) {
	commands
		.spawn(Camera2dBundle::default())
		.insert(bevy_pancam::PanCam::default());

	commands
		.spawn(Ball {
			pos: TransformBundle::from(Transform::from_translation(Vec3::new(0., 0., 0.))),
			visibility: InheritedVisibility::VISIBLE,
		})
		.with_children(|parent| {
			parent.spawn(MaterialMesh2dBundle {
				mesh: meshes.add(shape::Circle::new(10.).into()).into(),
				material: materials.add(ColorMaterial::from(Color::ORANGE)),
				..default()
			});
		});
}

#[derive(Clone, Debug, Eq, Hash, PartialEq, SystemSet)]
enum Set {
	Demo,
}

#[derive(Bundle)]
struct Ball {
	pos: TransformBundle,
	visibility: InheritedVisibility,
}

fn demo_system(query: Query<&Transform>, time: Res<Time>) {
	println!("Temps écoulé : {}s", time.delta_seconds());
	for pos in query.iter() {
		println!("Il y a un objet aux coordonnées {:?}", pos.translation);
	}
}
